'use strict';

angular.module('adf.widget.electre.lavoisier')
    .directive(
        'selectView',
        [
            
            "lavoisierHttpRequestService", "parserService", "LAVOISIER_TEMPLATES_PATH", "lavoisierViews",
            function (lavoisierHttpRequestService, parserService, LAVOISIER_TEMPLATES_PATH, lavoisierViews) {
                var templateUrl_;
                var lavoisier_views;
                /**
                 * This is called if the update function has failed.
                 * @param {type} element
                 * @param {type} scope
                 * @param {Error} e 
                 */
                function failToLoadViews (element, scope, e) {
                    scope.config.view = undefined;
                    scope.config.viewOk = false;
                    scope.errorMessage = e;

                    console.error(e || "Request error in selectView directive!");
                }
                /**
                 * Mets à jours la liste des vues.
                 * @param {type} scope
                 * @param {type} element
                 * @param {type} attrs
                 */
                function updateSelectView (scope, element, attrs) {
                    var _lavoisier_views = document.getElementById("lavoisier_views"), lavoisier_string = _lavoisier_views.innerHTML;
                    var lavoisier_json_config = parserService.xml_str2json(lavoisier_string), views = [];
                    /**
                     * @description Like a thread when using regular expressions in the view names config
                     * @type {Object}
                     */
                    var thread = Object.create(
                        {
                            /**
                             * @description Initialize the process by taking the list of view from the server
                             */
                            initialize: function () {
                                this.initializing = true;
                                var self = this;
                                var url_for_all_views_detail = (scope.config.baseurl || "/lavoisier")  + lavoisierViews;
                                lavoisierHttpRequestService.getResult({url : url_for_all_views_detail}).then(
                                    function (xml_views) {
                                        var lavoisier_json_views = parserService.xml_str2json(xml_views);
                                        lavoisier_views = [];
                                        if (lavoisier_json_views.entries.entry) {
                                            if (lavoisier_json_views.entries.entry.length) {
                                                for (var e = 0; e < lavoisier_json_views.entries.entry.length; e++)
                                                    lavoisier_views.push(lavoisier_json_views.entries.entry[e].__text);
                                            } else
                                                lavoisier_views.push(lavoisier_json_views.entries.entry.__text);
                                        } else {
                                            self.noViews = true;
                                        }
                                        self.initializing = false;
                                        self.initialized = true;
                                    }, function () {
                                        selectViewFailed(element, scope, "Cannot collect view name's list from server.");
                                    }
                                );
                            },
                            /**
                             * @description Start the process (after initializing ended - or mark the process as running_intended)
                             * @param scope {object}
                             * @param views {Array<String>|Array<string>}
                             */
                            start: function (scope, views) {
                                this.running_intended = true;
                                if (this.initialized && !this.running) {
                                    this.running_intended = false;
                                    this.running = true;
                                    delete this.addItem;
                                    if (!this.noViews) {
                                        for (var i = 0 ; i < this.items.length ; i++)
                                            views = getViews(views, this.items[i]);

                                        this.end(scope, views);
                                    } else if (this.onlyRegExp)
                                        selectViewFailed(element, scope, "No views from lavoisier.\nPlease make sure you have at least one view defined when using regular expressions.");
                                } else {
                                    this._ = { scope: scope, views: views };
                                }
                            },
                            /**
                             * @description Finish and close the process
                             * @param scope
                             * @param views
                             */
                            end: function (scope, views) {
                                if (this.running && !this.ended) {
                                    this.running = false;
                                    this.ended = true;

                                    selectViewSuccess(scope, views);
                                }
                            },
                            /**
                             * @description The process is personalized so this is to add an item to the process
                             * @param item {String|string} Add an item to the process
                             */
                            addItem: function (item) {
                                if (!this.used) this.initialize();
                                this.items.push(item);
                            }
                        },{
                            /**
                             * @description State that the process is initializing
                             * @type boolean
                             */
                            initializing: {
                                set: function (newValue) {
                                    if (typeof newValue === "boolean") {
                                        this._initializing = newValue;
                                        return true;
                                    }
                                    return false;
                                },
                                get: function () { return this._initializing||false; }
                            },
                            /**
                             * @description State that the process is initialized and ready to work
                             */
                            initialized: {
                                set: function (newValue) {
                                    if (typeof newValue === "boolean") {
                                        this._initialized = newValue;
                                        if (newValue && this.running_intended) this.start(this._.scope, this._.views);
                                        return true;
                                    }
                                    return false;
                                },
                                get: function () { return this._initialized||false; }
                            },
                            /**
                             * @description State that the process is running (the process can only run once
                             */
                            running: {
                                set: function (newValue) {
                                    if (typeof newValue === "boolean") {
                                        this._running=newValue;
                                        return true;
                                    }
                                    return false;
                                },
                                get: function () { return this._running||false; }
                            },
                            /**
                             * @description State that the process is ended when finished the process cannot be re-run
                             */
                            ended:{
                                set: function (newValue) {
                                    if (typeof newValue === "boolean") {
                                        this._ended=newValue;
                                        return true;
                                    }
                                    return false;
                                },
                                get: function () { return this._ended || false; }
                            },
                            /**
                             * @description The items the process uses when running
                             */
                            items: {
                                set: function (newValue) {
                                    if (typeof newValue === "object" && newValue.length !== undefined) {
                                        this._items = newValue;
                                        return true;
                                    }
                                    return false;
                                },
                                get: function () {
                                    if (!this._items) { this._items = []; }
                                    return this._items;
                                }
                            },
                            /**
                             * @description Tells if the thread is being used
                             */
                            used: {
                                get: function () { return this.initializing || this.initialized || this.running || this.running_intended || this.items.length > 0; }
                            },
                            /**
                             * @description State the intent to run the process while the process has not finished initializing phase
                             */
                            running_intended: {
                                value: false,
                                writable: true
                            },
                            /**
                             * @description Indicates that there is only regular expressions from configuration
                             */
                            onlyRegExp: {
                                value: false,
                                writable: true
                            },
                            /**
                             * @description Indicates that lavoisier return absolutely no view (at least no view that are accessible by regular expression)
                             */
                            noViews: {
                                value: false,
                                writable: true
                            }
                        }
                    );

                    try {
                        if (lavoisier_json_config.entries.entry.length) {
                            var onlyRegExp = true;
                            for (var e=0, entry=lavoisier_json_config.entries.entry[e]
                                ; e < lavoisier_json_config.entries.entry.length ;
                                 e++, entry=lavoisier_json_config.entries.entry[e]
                            ) {
                                var entry_key = entry._key;
                                if (entry_key.startsWith('/') && entry_key.endsWith('/')) {
                                    thread.addItem(entry_key);
                                } else {
                                    onlyRegExp = false;
                                    addString(views, entry_key, entry.__text === 'true', scope);
                                }
                            }
                            if (thread.used) thread.start(scope, views);
                        } else {
                            var entry_key = lavoisier_json_config.entries.entry._key;
                            if (entry_key.startsWith('/') && entry_key.endsWith('/')) {
                                thread.onlyRegExp = true;
                                thread.addItem(entry_key);
                            } else {
                                addString(views, entry_key, true, scope);
                            }
                            if (thread.used) thread.start(scope, views);
                        }

                        if (!thread.used) {
                            selectViewSuccess(scope, views)
                        }
                    } catch (e) {
                        selectViewFailed(element, scope, e);
                    }
                }

                /**
                 * @description Ajoute le nom d'une vue dans le tableau des vues.
                 * @param views {Array<String>|Array<string>} Tableau dans lequel ajouter la vue.
                 * @param viewName {String|string} Nom de la vue à ajouter.
                 * @param isdefault {boolean} Si la vue est pré-selectionné.
                 * @param scope {object} Pour définir la vue par défaut.
                 */
                function addString (views, viewName, isdefault, scope) {
                    if (!scope.config.view && isdefault) scope.config.view = viewName;
                    views.push(viewName);
                }
                /**
                 * @description Ajoute les vues correspondante à l'expression régulière dans le tableau.
                 * @param views {Array<String>|Array<string>} Tableau dans lequel ajouter la vue.
                 * @param regExp {String|string} Expression régulière à appliquer aux noms des vues.
                 */
                function getViews(views, regExp) {
                    var regex = new RegExp(regExp.substring(1, regExp.length - 1));
                    var filtered_views = lavoisier_views.filter(function (view) {
                        return regex.test(view);
                    });
                    return views.concat(filtered_views.filter(function (view) {
                        return views.indexOf(view) === -1;
                    }));
                }
                /**
                 * @description What to do if it's a success
                 * @param scope
                 * @param views
                 */
                function selectViewSuccess (scope, views) {
                    scope.views = views;
                    if (!scope.config.view) scope.config.view = views[0];

                    // Dernière vérification avant validation.
                    if (scope.views.length === 0) throw "Empty views in selectView directive!";
                    scope.config.viewOk = true;

                    templateUrl_ = "views_success.html";
                }
                /**
                 * @description What to do if it is a failure
                 * @param element
                 * @param scope
                 * @param e
                 */
                function selectViewFailed (element, scope, e) {
                    failToLoadViews(element, scope, e);
                    templateUrl_ = "views_failed.html";
                }

                return {
                    scope : {
                        control : '='
                    },
                    link : function (scope, element, attrs) {
                        scope.config = scope.control.config;
                        scope.control.controller.majViewContent = function () { updateSelectView(scope, element, attrs); };

                        if (!scope.control.controller.addEvent) {
                            angular.element(document.getElementById("lavoisier_url")).on("focusout", scope.control.controller.majViewContent);
                            scope.control.controller.addEvent = true;
                        }
                        
                        updateSelectView(scope, element, attrs);
                        
                        scope.getContentUrl  = function () {
                            if (templateUrl_) 
                                return LAVOISIER_TEMPLATES_PATH
                                    + "/edit/directives/" + templateUrl_;
                            else return false;
                            
                        };
                    },
                    template : "<div ng-include='getContentUrl()'></div>"
                };
            }
        ]
    );